/**
 * only for touch events (mobile)
 * need the dragged element has the style 'position:absolute', and css class 'drag' for method querySelectorAll('.drag');
 * @author zhaozhiyu
 */
(function (global, doc, factory) {
	factory = factory(global, doc, navigator.userAgent);
	//对外提供的接口
	if (typeof global.define === 'function' && (define.amd || define.cmd)) {
		define(function () {
			return factory;
		});
	} else {
		global.mTouch = global.mTouch || factory;
	}
})(window || this, document, function (global, doc, ua) {
	/**
	 * 相关控制配置项
	 * 
	 */
	var config = {
		dragX : true,								//是否响应横向拖拽,默认开启
		dragY : true,								//是否响应纵向拖拽,默认开启
		rowHasColumn: 1,							//每行有几列,默认每行1列
		dragableELsQuery : "",						//拖拽的元素，触发拖拽元素
		dragableRespondELsQuery:"",					//响应拖拽的元素，响应拖拽轨迹的元素
		bubbleCancel:function(target){return false;},//冒泡阶段不可响应的元素取消拖拽，在什么情况下要停止响应拖拽
		invalidRegions:[{x:0, y:0, width:0, height:0}],//配置不可拖拽区域
		onTouchStartCss:function(el){},				//参数为响应拖拽的元素	
		onTouchMoveCss:function(el){},				//参数为响应拖拽的元素
		onTouchEndCss:function(el){}				//参数为响应拖拽的元素
	};
	/**
	   * touch 设备
	   */
	var frame = { 
  		target:undefined,//目标元素对象
  		order:0,//排序从0开始
  		origin:{x : 0, y : 0}, //起点
  		size:{width : 0, height : 0},//宽高
  		offset:{x : 0, y : 0}//坐标偏移
  	   };//
	var MDrag = function(){
	}
	MDrag.prototype = {
		//配置drag事件相关控制的接口
		setConfig:function(cfg){
			for (var k in cfg) {
				if (cfg.hasOwnProperty(k)) {
					config[k] = cfg[k];
				}
			}
			var respondEL = getDragableRespondELs();
			frame.size.width = parseInt(respondEL[0].clientWidth);
			frame.size.height = parseInt(respondEL[0].clientHeight);
		},
		bindListener:function(){
			var dragELs = getDragableELs();
		    for (var i = 0; i < dragELs.length; i++) {
		    	dragELs[i].addEventListener('touchstart', handleTouchStart, false);
		    }
		},
		staticValues:function(){
			return {
				match_parent:0,//匹配父容器，即占满父容器（单指宽或者高，参照android的match_parent）
				other:null
			};
		},
		afterRemove:function(){
			frame.target = undefined;
		}
	};
	/**
	 * 响应拖拽的元素集合
	 */
	function getDragableRespondELs(){
		if(config.dragableRespondELsQuery == ""){
			return getDragableELs();
		}else{
			return document.querySelectorAll(config.dragableRespondELsQuery);
		}
	}
	/**
	 * 获取支持拖拽的元素集合
	 */
	function getDragableELs(){
		return document.querySelectorAll(config.dragableELsQuery);
	}
	/**
	 * 拖拽开始
	 */
	function handleTouchStart(e) {
		if(config.bubbleCancel(e.target)){
			e.target.parentElement.setAttribute("title", "");
			return;
		}
		
		var touch = e.touches[0];
		frame.target = getDragableRespondELs()[getDragoverELIndex(touch)];//
		frame.order = getDragoverELIndex(touch);
		frame.offset.x = frame.target.offsetLeft - touch.pageX;
		frame.offset.y = frame.target.offsetTop - touch.pageY;
		frame.target.style.zIndex = 10;
		frame.target.style.position = 'absolute';
		e.target.addEventListener('touchmove', handleTouchMove, false);
		e.target.addEventListener('touchend', handleTouchEnd, false);
		//e.preventDefault();
		config.onTouchStartCss(frame.target);
	}
	function handleTouchMove(e){
		if(config.bubbleCancel(e.target)){
			return;
		}
		/*if(e.target != frame.target){
			return;
		}*/
		var touch = e.changedTouches[0]; // NEED this to be separate from above.
		var dragOverEL = handleDragOver(touch);
		frame.target = e.target.parentElement;
		frame.target.style.zIndex = 11;
		if(config.dragX){
			frame.target.style.left = touch.pageX + frame.offset.x + 'px';
		}
		if(config.dragY){
			frame.target.style.top = touch.pageY + frame.offset.y + 'px';
		}
		config.onTouchMoveCss(frame.target);
	}
	function handleTouchEnd(e){
		if(config.bubbleCancel(e.target)){
			return;
		}
		/*if(e.target != frame.target){
			return;
		}*/
		var touch = e.changedTouches[0]; // NEED this to be separate from above.
		var dropEL = handleDrop(touch);
		frame.target.style.removeProperty('zIndex');
		frame.target.style.removeProperty('position');
		frame.target.style.removeProperty('left');
		frame.target.style.removeProperty('top')
		config.onTouchEndCss(frame.target);
	}
	/**
	 * 拖拽划过的元素处理
	 */
	function handleDragOver(touch){
		//TODO 可以增减划过时的特效
		return getDragoverEL(touch);
	}
	/**
	 * 拖拽后处理
	 */
	function handleDrop(touch){
		var dropEL = getDragoverEL(touch);
		frame.target.style.zIndex = 0;
		if(dropEL != undefined){
			//TODO 排序 保存请求
			if(frame.order>getDragoverELIndex(touch)){//向前拖拽
				$(dropEL).parent().before($(frame.target).parent());
			}else if(frame.order<getDragoverELIndex(touch)){//向后拖拽
				$(dropEL).parent().after($(frame.target).parent());
			}else{
				//未动
			}
		}
		return dropEL;
	}
	/*
	 * 获取，从0开始，一排3列
	 * 暂只支持顶部无效区域
	 */
	function getDragoverELIndex(touch){
		var i = 0, len = config.invalidRegions.length, index = 0;
		for (; i < len; i++) {
			if(index != -1){
				if(touch.pageY >= config.invalidRegions[i].y && touch.pageY <= (config.invalidRegions[i].y + config.invalidRegions[i].height)){
					if(touch.pageX >= config.invalidRegions[i].x && touch.pageX <= (config.invalidRegions[i].x + config.invalidRegions[i].width)){
						index = -1;
					}else{
						var row = Math.floor((touch.pageY-config.invalidRegions[i].height) / frame.size.height);
						var column = Math.floor((touch.pageX-config.invalidRegions[i].height) / frame.size.width);
						index = row * config.rowHasColumn + column;
					}
				}else{
					var row = Math.floor((touch.pageY-config.invalidRegions[i].height) / frame.size.height);
					var column = Math.floor(touch.pageX / frame.size.width);
					index = row * config.rowHasColumn + column;
				}
			}
		}
		return index;
	}
	/**
	 * 获取拖拽至目标元素
	 */
	function getDragoverEL(touch){
		var respondELs = getDragableRespondELs();
		var dragableELs = respondELs;
		return dragableELs[getDragoverELIndex(touch)];
	}
	/**
	 * 数据类型判断工具类库
	 */
	function TypeUtils(){
		var typeJs = {};
		typeJs.type = function(v){
	        return Object.prototype.toString.call(v);
	    };
	    /**
	     * 是否为数组对象类型  如果是就返回true 如果不是就返回false
	     * @namespace xjo.type
	     * @method isArray
	     * @param {Any} v 被检测的变量
	     * @return {Boolean} 结果
	     */
	    typeJs.isArray = function(v){
	        return typeJs.type(v) === '[object Array]';
	    };
	    typeJs.isNodeList = function(v){
	        return typeJs.type(v) === '[object NodeList]';
	    };
	    return typeJs;
	}
	//单例模式
	var mDrag = function () {
		return new MDrag();
	};
	window.mDrag = mDrag;//非AMD
	return mDrag;//AMD
});